/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;

uniform vec4 u_GUI_Position;
uniform vec4 u_GUI_ClipRect;
uniform vec4 u_GUI_Tiling;

out vec2 TexCoord;

void main()
{
   float X1 = u_GUI_Position.x;
   float Y1 = u_GUI_Position.y;
   float X2 = u_GUI_Position.z;
   float Y2 = u_GUI_Position.w;

   float Width  = X2 - X1;
   float Height = Y2 - Y1;

   vec4 VertexPos = vec4( X1 + in_Vertex.x * Width,
                          Y1 + in_Vertex.y * Height,
                          in_Vertex.z, in_Vertex.w );

   gl_Position = in_ModelViewProjectionMatrix * VertexPos;

	vec2 UV0 = u_GUI_ClipRect.xy;
	vec2 UV1 = u_GUI_ClipRect.zw;

	// wrap tiled coords
	vec2 ut = in_TexCoord.xy * u_GUI_Tiling.xy;

	vec2 umod = mod( ut, 1.0 );
	vec2 ustep = step( 0.0, ut );

	ut = ustep * ut + ( 1.0 - ustep ) * umod;

	// remap to clip rect
	vec2 uv = ( vec2(1.0) - ut ) * UV0 + ut * UV1;

   TexCoord = uv;
}

/*FRAGMENT_PROGRAM*/

in vec2 TexCoord;

uniform sampler2D Texture0;

uniform vec4 u_GUI_OpacityLifeTime;

out vec4 out_FragColor;

void main()
{
   out_FragColor = texture( Texture0, TexCoord ) * vec4( u_GUI_OpacityLifeTime.x );
//	out_FragColor = vec4( TexCoord.x, 0.0, 0.0, 1.0 );
}
