/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;

out vec2 v_TexCoord;

void main()
{
   gl_Position = in_ModelViewProjectionMatrix * in_Vertex;

	v_TexCoord = vec2( in_Vertex.x, 1.0-in_Vertex.y );
}

/*FRAGMENT_PROGRAM*/

uniform vec4  u_Resolution;       // viewport resolution (in pixels) in .xy and inverse resolution in .zw
uniform float u_TransitionValue;
uniform float u_PixelSize;

uniform sampler2D Texture0;      
uniform sampler2D Texture1;      

in vec2 v_TexCoord;

out vec4 out_FragColor;

void main(void)
{
	float T = u_TransitionValue;

	float S0 = 1.0;
	float S1 = u_PixelSize;
	float S2 = 1.0;

	// 2 segments, 1/2 each
	float Half = 0.5;

	float PixelSize = ( T < Half ) ? mix( S0, S1, T / Half ) : mix( S1, S2, (T-Half) / Half );

	vec2 D = PixelSize * u_Resolution.zw;

	// remap UV from 0...1 to -0.5...+0.5 to make the mosaic pattern converge torwards the image center
	vec2 UV = ( v_TexCoord + vec2( -0.5 ) ) / D;

	// don't forget to remap coords back to 0...1 after ceil()
	vec2 Coord = clamp( D * ( ceil( UV + vec2( -0.5 ) ) ) + vec2( 0.5 ), vec2( 0.0 ), vec2( 1.0 ) );

	vec4 C0 = texture( Texture0, Coord );
	vec4 C1 = texture( Texture1, Coord );

	out_FragColor = mix( C0, C1, T );
}
