/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;

out vec2 v_TexCoord;

void main()
{
   gl_Position = in_ModelViewProjectionMatrix * in_Vertex;

	v_TexCoord = vec2( in_Vertex.x, 1.0-in_Vertex.y );
}

/*FRAGMENT_PROGRAM*/

uniform vec4  u_Resolution;		// viewport resolution (in pixels) in .xy and inverse resolution in .zw
uniform float u_TransitionValue;
uniform float u_PixelSize;			// use 0.5 for a good default value

uniform sampler2D Texture0;      
uniform sampler2D Texture1;      

in vec2 v_TexCoord;

out vec4 out_FragColor;

void main(void)
{
	float Radius = u_PixelSize;

	float T = u_TransitionValue;

	vec2 UV = v_TexCoord.xy;

	UV -= vec2( 0.5, 0.5 );

	float Dist = length(UV);

	if ( Dist < Radius )
	{
		float Percent = (Radius - Dist) / Radius;
		float A = ( T <= 0.5 ) ? mix( 0.0, 1.0, T/0.5 ) : mix( 1.0, 0.0, (T-0.5)/0.5 );
		float Theta = Percent * Percent * A * 8.0 * 3.14159;
		float S = sin( Theta );
		float C = cos( Theta );
		UV = vec2( dot(UV, vec2(C, -S)), dot(UV, vec2(S, C)) );
	}
	UV += vec2( 0.5, 0.5 );

	vec4 C0 = texture2D( Texture0, UV );
	vec4 C1 = texture2D( Texture1, UV );

	out_FragColor = mix( C0, C1, T );
}
