/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;

out vec2 v_TexCoord;

uniform vec4 u_DiffuseColor;

uniform vec4 u_GUI_ClipRect;
uniform vec4 u_GUI_Tiling;

void main()
{
	// check out http://www.geeks3d.com/20140807/billboarding-vertex-shader-glsl/

	vec3 X = vec3(in_ModelViewMatrix[0][0], in_ModelViewMatrix[1][0], in_ModelViewMatrix[2][0]);
	vec3 Y = vec3(in_ModelViewMatrix[0][1], in_ModelViewMatrix[1][1], in_ModelViewMatrix[2][1]);
  
	float SizeX = u_DiffuseColor.x;
	float SizeY = u_DiffuseColor.y;

	vec3 XOfs = ( 2.0 * (in_TexCoord.x-0.5) * SizeX ) * X;
	vec3 YOfs = ( 2.0 * (in_TexCoord.y-0.5) * SizeY ) * Y;

	vec3 Position = XOfs + YOfs;

	vec4 TransformedPos = in_ModelViewProjectionMatrix * vec4( Position, 1.0 );

	gl_Position = TransformedPos;

	vec2 UV0 = u_GUI_ClipRect.xy;
	vec2 UV1 = u_GUI_ClipRect.zw;

	// wrap tiled coords
	vec2 ut = in_TexCoord.xy * u_GUI_Tiling.xy;

	// flip vertival
	ut = vec2( ut.x, 1.0 - ut.y );

	vec2 umod = mod( ut, 1.0 );
	vec2 ustep = step( 0.0, ut );

	ut = ustep * ut + ( 1.0 - ustep ) * umod;

	// remap to clip rect
	vec2 uv = ( vec2(1.0) - ut ) * UV0 + ut * UV1;

	v_TexCoord = uv;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 v_TexCoord;

uniform sampler2D Texture1; // ambient map

out vec4 out_FragColor;
                                              
void main()
{
	vec4 Color = texture( Texture1, v_TexCoord );

	out_FragColor = Color;
}
