/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;		// index, 0, thicknexx
in vec4 in_TexCoord;		// u, v
in vec3 in_Normal;		// line start
in vec3 in_Binormal;		// line end
in vec4 in_Color;			// color

out float v_Thickness;
out vec2 v_TexCoord;
out vec4 v_Color;

#include MaterialSystem/ShaderLib/Uniforms_Vertex.sp

void main()
{
	vec4 LineStart  = vec4( in_Normal, 1.0 );
	vec4 LineEnd    = vec4( in_Binormal, 1.0 );
	float Thickness = in_Vertex.z;

	mat4 MVP = in_ProjectionMatrix * in_ModelViewMatrix;

	vec4 Start = MVP * LineStart;
	vec4 End   = MVP * LineEnd;

	vec4 D = Thickness * normalize(End - Start);
	vec4 A = vec4( -D.y, D.x, 0.0, 0.0 );

	int Index = int( in_Vertex.x );

	vec4 Position = vec4(0);
	Position = ( (Index < 2) || ( Index > 5) ) ? Start : End;
	Position += (Index == 0 || Index == 7) ? -D : ( (Index == 3 || Index == 4) ? D : vec4(0) );
	Position += (Index < 4) ? A : -A;

	v_Thickness = Thickness;
	v_TexCoord = in_TexCoord.xy;
	v_Color = in_Color;

	gl_Position = Position;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in float v_Thickness;
in vec2 v_TexCoord;
in vec4 v_Color;

out vec4 out_FragColor;

#include MaterialSystem/ShaderLib/Uniforms_Fragment.sp

void main()
{
	float Thickness = v_Thickness;

	float Coef = Thickness / 3.0;

	vec2 Dist  = ( v_TexCoord - vec2( 0.5 ) ) / Coef;
	vec2 Dist2 = -Dist*Dist;

	float Alpha = exp( Dist2.x + Dist2.y );

	Alpha = smoothstep( 0.1, 0.1, Alpha ) * v_Color.w;

	if (Alpha < 0.005) discard;

	vec4 FinalColor = vec4( v_Color.xyz, Alpha );

	out_FragColor = FinalColor;
}
