/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord; // TTL, lifetime, width, height
in vec4 in_Color;

out vec2 v_Size;
out vec4 v_Color;

#include MaterialSystem/ShaderLib/Particles.sp

void main()
{
	gl_Position = in_Vertex;

	float TTL = in_TexCoord.x;
	float LifeTime = in_TexCoord.y;

	float NormalizedTime = ( LifeTime - TTL ) / LifeTime;

   v_Size  = vec2( in_TexCoord.z, in_TexCoord.w );
   v_Color = in_Color * GetParticleBrightness( NormalizedTime );
}

/*GEOMETRY_PROGRAM*/

#include Layout.sp

layout (points) in;
layout (triangle_strip, max_vertices = 4) out;

in vec2 v_Size[];
in vec3 v_Params[];
in vec4 v_Color[];

out vec2 g_TexCoord;
out vec4 g_Color;

uniform float u_ParticlesRenderMode;

void main()
{
	const vec2 Coords[4] = vec2[]
	(
		vec2(0.0, 1.0),
		vec2(0.0, 0.0),
		vec2(1.0, 1.0),
		vec2(1.0, 0.0)
	);

	vec3 X = vec3( in_ModelViewMatrix[0][0], in_ModelViewMatrix[1][0], in_ModelViewMatrix[2][0] );
	vec3 Y = vec3( in_ModelViewMatrix[0][1], in_ModelViewMatrix[1][1], in_ModelViewMatrix[2][1] );

	vec3 P = gl_in[0].gl_Position.xyz;

	vec3 OfsX = X * v_Size[0].x;
	vec3 OfsY = Y * v_Size[0].y;

	vec3 Ofs[4];

	Ofs[0] = OfsX * -1.0 + OfsY * -1.0;
	Ofs[1] = OfsX * -1.0 + OfsY *  1.0;
	Ofs[2] = OfsX *  1.0 + OfsY * -1.0;
	Ofs[3] = OfsX *  1.0 + OfsY *  1.0;

	// eParticleRenderMode_VerticalBillboard
	if ( u_ParticlesRenderMode > 0.5 )
	{
		// make parallel to the Z-axis
		Ofs[0].xy = Ofs[1].xy;
		Ofs[2].xy = Ofs[3].xy;
	}

	for ( int i = 0; i != 4; i++ )
	{
		gl_Position = in_ModelViewProjectionMatrix * vec4( P + Ofs[i], 1.0 );
		g_TexCoord = Coords[i];
		g_Color  = v_Color[0];
		EmitVertex();
	}

	EndPrimitive();
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 g_TexCoord;
in vec4 g_Color; /* RGBA overlay */

uniform sampler2D Texture1; /* Color texture */

out vec4 out_FragColor;
                                              
void main()
{
	vec4 ParticleColor = vec4( texture(Texture1, g_TexCoord) );

	out_FragColor = g_Color * ParticleColor;
}
