/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord; // TTL, lifetime, W, H
in vec4 in_Color;

out vec4 v_Color;

#include MaterialSystem/ShaderLib/Particles.sp

uniform float cvar_ViewportWidth;

void main()
{
	float Size = 2.0 * in_TexCoord.z;

	vec4 EyePos = in_ModelViewMatrix * in_Vertex;
	vec4 ProjectedSize = in_ProjectionMatrix * vec4( Size, Size, EyePos.z, EyePos.w );
	vec2 Pixels = cvar_ViewportWidth * ProjectedSize.xy / ProjectedSize.w;

	gl_PointSize = 0.5 * Pixels.x;
	gl_Position = in_ModelViewProjectionMatrix * in_Vertex;

	float TTL = in_TexCoord.x;
	float LifeTime = in_TexCoord.y;

	float NormalizedTime = ( LifeTime - TTL ) / LifeTime;

   v_Color = in_Color * GetParticleBrightness( NormalizedTime );
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec4 v_Color; /* RGBA overlay */

uniform sampler2D Texture1; /* Color texture */

out vec4 out_FragColor;
                                              
void main()
{
//	vec2 p = gl_PointCoord * 2.0 - vec2(1.0);
//	float theta = atan(p.y,p.x);
//	if(dot(p,p) > 0.05*(exp(cos(theta*5)*0.75)) ) discard;

	vec4 ParticleColor = vec4( texture(Texture1, gl_PointCoord) );

	out_FragColor = v_Color * ParticleColor;
}
