/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord; // TTL, lifetime, width, height
in vec3 in_Normal; // tex coords...
in vec4 in_Color;

out vec2 v_TexCoord;
out vec4 v_Params;
out vec4 v_Color;

#include MaterialSystem/ShaderLib/Particles.sp

uniform float u_ParticlesRenderMode;

void main()
{
   vec3 X = vec3(in_ModelViewMatrix[0][0], in_ModelViewMatrix[1][0], in_ModelViewMatrix[2][0]);
   vec3 Y = vec3(in_ModelViewMatrix[0][1], in_ModelViewMatrix[1][1], in_ModelViewMatrix[2][1]);
  
   float SizeX = in_TexCoord.z;
   float SizeY = in_TexCoord.w;

	// eParticleRenderMode_VerticalBillboard
	if ( u_ParticlesRenderMode > 0.5 )
	{
	   vec3 A1 = -X*SizeX - Y*SizeY;
	   vec3 B1 =  X*SizeX - Y*SizeY;
	   vec3 C1 =  X*SizeX + Y*SizeY;
	   vec3 D1 = -X*SizeX + Y*SizeY;

	   // make parallel to Z-axis
	   A1.x = D1.x;
	   B1.x = C1.x;
	   A1.y = D1.y;
	   B1.y = C1.y;

		vec3 Position = in_Vertex.xyz;

	   if ( in_Normal.xy == vec2(0.0, 1.0) ) Position += A1;
	   if ( in_Normal.xy == vec2(1.0, 1.0) ) Position += B1;
	   if ( in_Normal.xy == vec2(1.0, 0.0) ) Position += C1;
	   if ( in_Normal.xy == vec2(0.0, 0.0) ) Position += D1;

	   gl_Position = in_ModelViewProjectionMatrix * vec4( Position, 1.0 );

	}
	else
	{
		vec3 XOfs = ( 2.0 * (in_Normal.x-0.5) * SizeX ) * X;
		vec3 YOfs = ( 2.0 * (1.0-in_Normal.y-0.5) * SizeY ) * Y;

		vec3 Position = in_Vertex.xyz + XOfs + YOfs;

	   gl_Position = in_ModelViewProjectionMatrix * vec4( Position, 1.0 );
	}

	float TTL = in_TexCoord.x;
	float LifeTime = in_TexCoord.y;

	float NormalizedTime = ( LifeTime - TTL ) / LifeTime;

   v_TexCoord = in_Normal.xy;
   v_Params   = in_TexCoord;
   v_Color    = in_Color * GetParticleBrightness( NormalizedTime );
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 v_TexCoord;
in vec4 v_Params; /* TTL, LifeTime, W, H */
in vec4 v_Color; /* RGBA overlay */

uniform sampler2D Texture1; /* Color texture */

out vec4 out_FragColor;
                                              
void main()
{
   vec4 ParticleColor = vec4( texture(Texture1, v_TexCoord) );

   float TTL      = v_Params.x;
   float LifeTime = v_Params.y;

	out_FragColor = v_Color * ParticleColor;
}
