/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;

out vec2 TexCoord0;

void main()
{
   TexCoord0 = vec2( in_TexCoord );

   gl_Position = in_ModelViewProjectionMatrix * in_Vertex;
}

/*FRAGMENT_PROGRAM*/

in vec2 TexCoord0;

uniform sampler2D Texture0;     // Color
uniform sampler2D Texture1;     // Avg. luminance
uniform sampler2D Texture2;     // Bloom

uniform float BloomStrength;
uniform float MidGray;
uniform float White;

out vec4 out_FragColor;

// [John Hable] Uncharted 2: HDR Lighting
vec4 Uncharted2Tonemap( vec4 x )
{
	float A = 0.22;	// shoulder strength
	float B = 0.30;	// linear strength
	float C = 0.10;	// linear angle
	float D = 0.20;	// toe strength
	float E = 0.01;	// toe numerator
	float F = 0.30;	// toe denumerator

	// slide #142
	return ((x*(A*x+vec4(C*B))+vec4(D*E))/(x*(A*x+vec4(B))+vec4(D*F)))-vec4(E/F);
}

// [John Hable] Uncharted 2: HDR Lighting
vec4 ConvertToGamma( vec4 LinearColor )
{
	// slide #140
	vec4 x = max( vec4(0.0), LinearColor - vec4( 0.004 ) );
	return (x*(6.2*x+vec4(0.5)))/(x*(6.2*x+vec4(1.7))+vec4(0.06));
}

void main()
{
	float AvgLuminance = texture( Texture1, vec2(0.5, 0.5) ).x;

	vec4  Color  = vec4( texture(Texture0, TexCoord0) );
	vec4  Bloom  = vec4( texture(Texture2, TexCoord0) );

	float Exposure = MidGray / ( AvgLuminance + 0.001 );

	Color *= Exposure;

	// [John Hable] Uncharted 2: HDR Lighting
	// slide #143
	float W = 11.2;

	Color = Uncharted2Tonemap( Color ) / Uncharted2Tonemap( vec4(W) );

//	vec4 GammaColor = ConvertToGamma( Color );

/*
	// Reinhard tone mapping operator
	Color = Color * ( 1.0 + Color / (White*White) ) / ( 1.0 + Color );
*/
   out_FragColor = Color + BloomStrength * Bloom;

}
