/** Merged calc_alpha.frag and calc_alpha_optim.frag */

#include Layout.sp

out vec4 out_FragColor;

// TEX LGN COL CONTAINS DISCONTINUITY :
// R/G : vertical/horizontal discontinuity
uniform sampler2D tex_lgn_col;

// TEXTURE tex_distances CONTAINS (r,g,b,a) : left/right/up/bottom distance (number of pixels)
uniform sampler2D tex_distances;

//uniform sampler2D tex_aires;

in vec4 owncoordinates;

uniform vec4 coeff_image;
// RESULT : glFragColor(r,g,b,a) : alpha with up/down/left/right pixel

const float max_size = 128.;
const float max_size_on_2 = max_size/2.;

float CalcAire(float xx, float yy)
{
	int i = (int)( xx * 255.5 );
	int j = (int)( yy * 255.5 );

	int S = (i+1);
	int borne = (S % 2 == 0) ? (S/2)-1 : (S/2);

	// Cas triangle (et taille ligne paire (2) ou impaire (8))
	float Mult = (S % 2 == 0) ? 2.0f : 8.0f;

	float Val = 0.0f;

	if (j < borne) {
		// CAS TRAPEZE
		Val = 0.5f * (1.0f - (float)(2 * j + 1) / (float)S);
	}
	else
	{
		Val = (j == borne) ? (1.0f / (Mult * (float)S)) : 0.0f;
	}

	return Val;
}

vec2 get_aire_tex_coord_from_pos_and_size(float indice,float size_line) {
	const float size_texel_tex_aire = 1./256.;
	return vec2(0.5 + indice, -0.5 + size_line) * size_texel_tex_aire;
}

float get_value(float ToCheck, int Case, vec2 res_lgn_or_col)
{
	if(ToCheck <= 0) return 0;

	vec2 RG = res_lgn_or_col * 255.;
	RG -= vec2(1.);

	float R = RG.r;
	float G = RG.g;

	float size = R + G;

	// for 0 and 1
	if(Case == 0 || Case 1) size = min(size, max_size);

	float X = owncoordinates.x;
	float Y = owncoordinates.y;

	float TX = min(R, G);

	if(Case == 0) if (R <= G) { X -= (R + 1.) * coeff_image.x; } else { X += G * coeff_image.x; }
	if(Case == 1) if (R <= G) { Y += (R + 1.) * coeff_image.y; } else { Y -= G * coeff_image.y; }
	if(Case == 2) if (R <= G) { X -= (R + 1.) * coeff_image.x; } else { X += G * coeff_image.x; }
	if(Case == 3) if (R <= G) { Y += (R + 1.) * coeff_image.y; } else { Y -= G * coeff_image.y; }

	vec4 CompV = texture2D(tex_lgn_col,vec2(X,Y));

	float Comp;

	if(Case == 0 || Case == 2) Comp = CompV.r;
	if(Case == 1 || Case == 3) Comp = CompV.g;

	// first check is for 0 and 1
	if( (Case == 0 || Case == 1) && (TX > max_size_on_2)) return 0.;

//	return (Comp > 0) ? texture2D(tex_aires,get_aire_tex_coord_from_pos_and_size(TX,size)).r : 0;
	return (Comp > 0) ? CalcAire(tex_aires,get_aire_tex_coord_from_pos_and_size(TX,size)) : 0;
}

/// LV : unfortunately, we do not support defines. So the tex_distances/tex_lgns/lex_cols are cosen manually four times
void main() {
	vec4 line_center = texture2D(tex_lgn_col,owncoordinates.xy);
	// CAS |_| (en dessous)

	out_FragColor.g = get_value(line_center.g, 0, texture2D(tex_distances,owncoordinates.xy).rg);

	// CAS _
	//     _|
	out_FragColor.b = get_value(line_center.r, 1, texture2D(tex_distances,owncoordinates.xy).ba);

	// CAS _
	//    | |
	line_center = texture2D(tex_lgn_col,gl_TexCoord[0].zw);
	out_FragColor.r = get_value(line_center.g, 2, texture2D(tex_distances,gl_TexCoord[0].zw).rg);

	// CAS _
	//    |_
	line_center = texture2D(tex_lgn_col,gl_TexCoord[1].xy);
	out_FragColor.a = get_value(line_center.r, 3, texture2D(tex_distances,gl_TexCoord[1].xy).ba);
}
