/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;

out vec3 v_WorldPos;
out vec2 v_TexCoord;

void main()
{
   gl_Position = in_ModelViewProjectionMatrix * in_Vertex;
	
	v_WorldPos = ( in_ModelViewMatrix * in_Vertex ).xyz;
   v_TexCoord = vec2( in_TexCoord );
}


/*FRAGMENT_PROGRAM*/

uniform sampler2D Texture0;   // Color
uniform sampler2D Texture1;   // Depth
uniform sampler2D Texture2;   // Rotations map

in vec3 v_WorldPos;
in vec2 v_TexCoord;

out vec4 out_FragColor;

// zNear, zFar
uniform vec4 u_SSAOProjectionParams;

// Radius, AttScale, DistScale
uniform vec4 u_SSAOParams;

// based on http://steps3d.narod.ru/tutorials/ssao-tutorial.html
void main()
{
	float zNear     = u_SSAOProjectionParams.x;
	float zFar      = u_SSAOProjectionParams.y;
   float Radius    = u_SSAOParams.x;
	float AttScale  = u_SSAOParams.y;
	float DistScale = u_SSAOParams.z;
    
	vec4 RndTable[8] = vec4[8] 
	(
		vec4( -0.5, -0.5, -0.5, 0.0 ),
		vec4(  0.5, -0.5, -0.5, 0.0 ),
		vec4( -0.5,  0.5, -0.5, 0.0 ),
		vec4(  0.5,  0.5, -0.5, 0.0 ),
		vec4( -0.5, -0.5,  0.5, 0.0 ),
		vec4(  0.5, -0.5,  0.5, 0.0 ),
		vec4( -0.5,  0.5,  0.5, 0.0 ),
		vec4(  0.5,  0.5,  0.5, 0.0 )
	);

	float zFarMulzNear   = zFar * zNear;
	float zFarMinuszNear = zFar - zNear;
    
	// get Z in eye space
	float Z     = zFarMulzNear / ( texture( Texture1, v_TexCoord ).x * zFarMinuszNear - zFar );
	float Att   = 0.0;
	vec3  Plane = 2.0 * texture( Texture2, v_TexCoord * 512.0 / 4.0 ).xyz - vec3( 1.0 );
  
	for ( int i = 0; i < 8; i++ )
	{
		vec3  rSample = reflect( RndTable[i].xyz, Plane );
		float zSample = texture( Texture1, v_TexCoord + Radius*rSample.xy / Z ).x;

		zSample = zFarMulzNear / ( zSample * zFarMinuszNear - zFar );
        
		float Dist = max( zSample - Z, 0.0 ) / DistScale;
		float Occl = 15.0 * max( Dist * (2.0 - Dist), 0.0 );
        
		Att += 1.0 / ( 1.0 + Occl*Occl );
	}
    
	Att = clamp( Att * Att / 64.0 + 0.45, 0.0, 1.0 ) * AttScale;

	out_FragColor = vec4 ( Att, Att, Att, 1.0 );
}
