/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;
in vec3 in_Normal;
in vec2 in_Bones;

out vec2 v_TexCoord;
out vec4 v_ProjectedVertex;

#include MaterialSystem/ShaderLib/Uniforms_Vertex.sp
#include MaterialSystem/ShaderLib/Projections.sp

#if defined( SUPPORTED_VERTEX_TEXTURE_FETCH )
#include MaterialSystem/ShaderLib/QuaternionLib.sp
#include MaterialSystem/ShaderLib/Skinning.sp
#endif

void main()
{
#if defined( SUPPORTED_VERTEX_TEXTURE_FETCH )
   vec4 Position = ( u_KeyFrame.w == 1.0 ) ? SkinVertex() : in_Vertex;
#else
   vec4 Position = in_Vertex;
#endif

   gl_Position = in_ModelViewProjectionMatrix * Position;

   v_TexCoord = in_TexCoord.xy;
	v_ProjectedVertex = GetProjScaleBiasMat() * in_ModelViewProjectionMatrix * Position;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 v_TexCoord;
in vec4 v_ProjectedVertex;

out vec4 out_FragColor;

#include MaterialSystem/ShaderLib/Uniforms_Fragment.sp

void main()
{
	vec4 AmbientTexture = texture( Texture1, v_TexCoord );

	if ( u_MaterialAlphaTest.x > 0.0 )
	{
		if ( AmbientTexture.a < u_MaterialAlphaTest.x ) discard;
	}

	vec4 FinalColor = AmbientTexture + u_AmbientColor;

	float Reflectivity = u_MaterialParams.y;

	if ( Reflectivity > 0.0 )
	{
		FinalColor += textureProj( Texture5, v_ProjectedVertex ) * Reflectivity;
	}
	
	FinalColor.a = 1.0;

	out_FragColor = FinalColor;
}
