/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;
in vec3 in_Normal;
in vec2 in_Bones;

out vec2 v_TexCoord;
out float v_ClipDistance;

#include MaterialSystem/ShaderLib/Uniforms_Vertex.sp
#include MaterialSystem/ShaderLib/Projections.sp

#if defined( SUPPORTED_VERTEX_TEXTURE_FETCH )
#include MaterialSystem/ShaderLib/QuaternionLib.sp
#include MaterialSystem/ShaderLib/Skinning.sp
#endif

void main()
{
#if defined( SUPPORTED_VERTEX_TEXTURE_FETCH )
   vec4 Position = ( u_KeyFrame.w == 1.0 ) ? SkinVertex() : in_Vertex;
#else
   vec4 Position = in_Vertex;
#endif

	gl_Position = in_ModelViewProjectionMatrix * Position;

	v_TexCoord = in_TexCoord.xy;

	v_ClipDistance = dot( u_ClipPlane, in_ModelMatrix * Position );
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 v_TexCoord;
in float v_ClipDistance;

out vec4 out_FragColor;

uniform vec4 u_AmbientColor;
uniform vec4 u_DiffuseColor;

uniform sampler2D Texture1; // ambient map
uniform sampler2D Texture2; // diffuse map

void main()
{
	if ( v_ClipDistance < 0.0 ) discard;

	vec4 Ka = texture( Texture1, v_TexCoord ) + u_AmbientColor;
	vec4 Kd = texture( Texture2, v_TexCoord ) + u_DiffuseColor;

	out_FragColor = Kd;
}
