/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;
in vec3 in_Normal;
in vec2 in_Bones;

out vec2 v_TexCoord;
out vec3 v_WorldPosition;

uniform sampler2D Texture0; // packed skeleton
uniform vec4 u_KeyFrame;    // Frame1, Frame2, Lerp, Skinned

#if defined( SUPPORTED_VERTEX_TEXTURE_FETCH )
#include MaterialSystem/ShaderLib/QuaternionLib.sp
#include MaterialSystem/ShaderLib/Skinning.sp
#endif

void main()
{
#if defined( SUPPORTED_VERTEX_TEXTURE_FETCH )
   vec4 Position = ( u_KeyFrame.w == 1.0 ) ? SkinVertex() : in_Vertex;
#else
   vec4 Position = in_Vertex;
#endif

   gl_Position = in_ModelViewProjectionMatrix * Position;

	v_TexCoord = in_TexCoord.xy;
	v_WorldPosition = ( in_ModelMatrix * Position ).xyz;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 v_TexCoord;
in vec3 v_WorldPosition;

out vec4 out_FragColor;

#include MaterialSystem/ShaderLib/Uniforms_Fragment.sp
#include MaterialSystem/ShaderLib/Pack.sp

void main()
{
	// alpha test
	if ( u_MaterialAlphaTest.x > 0.0 )
	{
		if ( texture( Texture2, v_TexCoord ).a < u_MaterialAlphaTest.x ) discard;
	}

	float D = distance( v_WorldPosition, u_LightPosition.xyz );

	// we have 8-bits per channel on GLES3, just improvise
	out_FragColor = Pack( D / 512.0 );
}
