//
// Lighting computations
//

// L - normalized vector, pointing from the surface to the light source
float ComputeDiffuseTerm( vec3 L, vec3 Normal )
{
	return clamp( dot( L, Normal ), 0.0, 1.0 );
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputeDirectionalLight_Lambert( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	float Kd = ComputeDiffuseTerm( -u_LightDirection.xyz, Normal );

	return vec4( Kd ) * u_LightColor * Diffuse;
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputePointLight_Lambert( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 LightDirection = normalize( v_WorldPosition - u_LightPosition.xyz );

	float Kd = ComputeDiffuseTerm( -LightDirection.xyz, Normal );

	float D = length( u_LightPosition.xyz - v_WorldPosition );

	float AttenuationCoef = AttenuationForLight( D );

	return vec4( Kd ) * u_LightColor * Diffuse * AttenuationCoef;
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputeSpotLight_Lambert( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 LightDirection = normalize( v_WorldPosition - u_LightPosition.xyz );

	float InnerAngle = u_LightSpotRange.x;
	float OuterAngle = u_LightSpotRange.y;
	float Falloff    = u_LightSpotRange.z;

	float Rho = max( 0.0, dot( normalize( LightDirection ), u_LightDirection.xyz ) );

	vec4 Color = vec4( 0.0 );

	if ( Rho > OuterAngle )
	{
		float SpotFactor = pow( (Rho-OuterAngle) / (InnerAngle-OuterAngle), Falloff );
		Color = ComputePointLight_Lambert( Normal, Diffuse, Specular ) * SpotFactor;
	}

	return Color;
}
