/*
	Light - normalized surface-to-light vector
	Eye   - normalized surface-to-eye vector
*/

vec4 ComputeMinnaertTerm( vec3 Light, vec3 Eye, vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	// roughness
	float K = u_MaterialShading.z;

	float D1 = pow( max ( dot ( Normal, Light ), 0.0 ), 1.0 + K );
	float D2 = pow( 1.0 - dot ( Normal, Eye ), 1.0 - K );

	return vec4( D1 * D2 ) * u_LightColor * Diffuse;
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputeDirectionalLight_Minnaert( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 CameraPos = in_ViewMatrixInverse[3].xyz;

	vec3 Light = normalize( -u_LightDirection.xyz );
	vec3 Eye   = normalize( CameraPos - v_WorldPosition );

	return ComputeMinnaertTerm( Light, Eye, Normal, Diffuse, Specular );
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputePointLight_Minnaert( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 CameraPos = in_ViewMatrixInverse[3].xyz;

	vec3 Light = normalize( u_LightPosition.xyz - v_WorldPosition );
	vec3 Eye   = normalize( CameraPos - v_WorldPosition );

	// attenuation
	float AttenuationCoef = AttenuationForLight( length( u_LightPosition.xyz - v_WorldPosition ) );

	return ComputeMinnaertTerm( Light, Eye, Normal, Diffuse, Specular ) * AttenuationCoef;
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputeSpotLight_Minnaert( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 LightDirection = normalize( v_WorldPosition - u_LightPosition.xyz );

	float InnerAngle = u_LightSpotRange.x;
	float OuterAngle = u_LightSpotRange.y;
	float Falloff    = u_LightSpotRange.z;

	float Rho = max( 0.0, dot( normalize( LightDirection ), u_LightDirection.xyz ) );

	vec4 Color = vec4( 0.0 );

	if ( Rho > OuterAngle )
	{
		float SpotFactor = pow( (Rho-OuterAngle) / (InnerAngle-OuterAngle), Falloff );
		Color = ComputePointLight_Minnaert( Normal, Diffuse, Specular ) * SpotFactor;
	}

	return Color;
}
