/*
	Light - normalized surface-to-light vector
	Eye   - normalized surface-to-eye vector
*/

vec4 ComputeOrenNayarTerm( vec3 Light, vec3 Eye, vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	float Roughness = u_MaterialShading.z;

	float Cos_Theta_i = dot( Light, Normal );

	float V_dot_N = dot( Eye, Normal );

	float Theta_r = acos( V_dot_N );
	float Theta_i = acos( Cos_Theta_i );

	float Cos_Phi_Diff = dot( normalize( Eye - Normal * V_dot_N ), normalize( Light - Normal * Cos_Theta_i ) );
	float Max_Zero_Cos_Phi_Diff = max( 0.0, Cos_Phi_Diff );

	float Roughness2 = Roughness * Roughness + 0.00001;

	float A = 1.0 - 0.5 * (Roughness2 / (Roughness2 + 0.33));
	float B = 0.45 * (Roughness2 / (Roughness2 + 0.09));

	float Alpha = max( Theta_i, Theta_r );
	float Beta  = min( Theta_i, Theta_r );

	float Kd = Cos_Theta_i * (A + (B * Max_Zero_Cos_Phi_Diff * sin(Alpha) * tan(Beta) ) );

	return vec4( Kd ) * u_LightColor * Diffuse;
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputeDirectionalLight_OrenNayar( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 CameraPos = in_ViewMatrixInverse[3].xyz;

	vec3 Light = normalize( -u_LightDirection.xyz );
	vec3 Eye   = normalize( CameraPos - v_WorldPosition );

	return ComputeOrenNayarTerm( Light, Eye, Normal, Diffuse, Specular );
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputePointLight_OrenNayar( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 CameraPos = in_ViewMatrixInverse[3].xyz;

	vec3 Light = normalize( u_LightPosition.xyz - v_WorldPosition );
	vec3 Eye   = normalize( CameraPos - v_WorldPosition );

	// attenuation
	float AttenuationCoef = AttenuationForLight( length( u_LightPosition.xyz - v_WorldPosition ) );

	return ComputeOrenNayarTerm( Light, Eye, Normal, Diffuse, Specular ) * AttenuationCoef;
}

#if defined( USE_OPENGL_4 )
subroutine(ShadingModelTerm)
#endif
vec4 ComputeSpotLight_OrenNayar( vec3 Normal, vec4 Diffuse, vec4 Specular )
{
	vec3 LightDirection = normalize( v_WorldPosition - u_LightPosition.xyz );

	float InnerAngle = u_LightSpotRange.x;
	float OuterAngle = u_LightSpotRange.y;
	float Falloff    = u_LightSpotRange.z;

	float Rho = max( 0.0, dot( normalize( LightDirection ), u_LightDirection.xyz ) );

	vec4 Color = vec4( 0.0 );

	if ( Rho > OuterAngle )
	{
		float SpotFactor = pow( (Rho-OuterAngle) / (InnerAngle-OuterAngle), Falloff );
		Color = ComputePointLight_OrenNayar( Normal, Diffuse, Specular ) * SpotFactor;
	}

	return Color;
}
