// parallax occlusion mapping

float GetTextureHeight( vec2 Coord )
{
	return texture( Texture4, Coord ).a;
}

vec3 GetTextureNormal( vec2 Coord )
{
	return normalize( 2.0 * texture( Texture4, Coord ).xyz - vec3( 1.0 ) );
}

float CalculateParallaxOcclusionMapping( inout vec2 TexCoord, bool OcclusionMapping )
{
	// parallax mapping is done in tangent space
	vec3 n = normalize(v_EyeNormal);
	vec3 t = normalize(v_EyeTangent);
	vec3 b = normalize(v_EyeBinormal);

	vec3 p = (in_ModelViewMatrix * v_Position).xyz;
	vec3 e = normalize( vec3(0.0) - p );
	vec3 et = t * e.x + b * e.y + n * e.z;

	float	MinSteps  = u_ParallaxParams.x;
	float MaxSteps  = u_ParallaxParams.y;
	float	BumpScale = u_ParallaxParams.z;
	const float	Eps       = 0.05;

	float NumSteps  = MaxSteps + et.z * ( MinSteps - MaxSteps );
	float	Step   = 1.0 / NumSteps;
	vec2	dTex   = Step * BumpScale * et.xy / et.z;	// adjustment for one layer
	float	Height = 0.99;										// height of the layer
	vec2	Tex    = TexCoord;								// initial guess
	float	H      = 1.0 - GetTextureHeight( Tex );

	while ( H < Height )
	{
		Height -= Step;
		Tex    += dTex;
		H       = 1.0 - GetTextureHeight( Tex );
	}

	// now find point via linear interpolation
	vec2	Prev   = Tex - dTex;
	float	hPrev  = 1.0 - GetTextureHeight( Prev ) - ( Height + Step );
	float	hCur   = H - Height;
	
	Tex = mix( Tex, Prev, hCur / ( hCur - hPrev ) );

	TexCoord = Tex;

	float	Vis = 1.0;

	if ( OcclusionMapping )
	{
		vec3 l = normalize( p - (in_ViewMatrix * u_LightPosition).xyz );
		vec3 lt = t * l.x + b * l.y + n * l.z;
	
		Height    = 1.0 - GetTextureHeight( Tex );
		Step      = GetTextureHeight( Tex ) / NumSteps;
		vec2 lTex = BumpScale * Step * lt.yx / lt.z;
		Height   += Step;
		Tex      += lTex;

		float	SampleWeight = 1.0 / NumSteps;
		
		while ( Height < 1.0 - Eps )
		{
			if ( 1.0 - GetTextureHeight( Tex ) > Height ) Vis -= SampleWeight;
			Height += Step;
			Tex    += lTex;
		}
	}

	return Vis;
}
