// volumetric light scattering

vec4 ComputeVolumetricScatteringSpot()
{
	vec3 CameraPos = in_ViewMatrixInverse[3].xyz;
	vec3 WorldPos  = v_WorldPosition;
	vec3 Ray = WorldPos-CameraPos;

	float Steps = 128.0;
	float StepLength = length( Ray ) / Steps;
	vec3 StepVec = Ray / Steps;
	vec4 InScattering = vec4( 0.0 );

	vec3  CurPoint = CameraPos;
	float CurDist  = 0.0;
	float Albedo = 10.0;
	float Tau = 0.02;
	float Phi = 32.0;
	mat4 Transform = in_ShadowMatrix[0];
	for ( int i = 0; i != int(Steps); i++ )
	{
		CurPoint += StepVec;
		CurDist  += StepLength;
		vec4 ShadowMapCoord = Transform * vec4( CurPoint, 1.0 );
		vec4 ShadowCoords4 = ShadowMapCoord / ShadowMapCoord.w;
		if ( ShadowCoords4.w > 0.0 )
		{
			// point on the ray is not in shadow
			if ( ShadowCoords4.z < texture( Texture7, ShadowCoords4.xy ).x )
			{
				float DistToLight = length( u_LightPosition.xyz - CurPoint );
				float Lin = exp( -DistToLight * Tau ) * Phi/4.0/3.14159/DistToLight/DistToLight;
				float Li  = Lin * Tau * Albedo;
				InScattering += Li * exp( -CurDist * Tau ) * CurDist;
			}
		}
	}

	return InScattering;
}

vec4 ComputeVolumetricScatteringOmni()
{
	vec3 CameraPos = in_ViewMatrixInverse[3].xyz;
	vec3 WorldPos  = v_WorldPosition;
	vec3 Ray = WorldPos-CameraPos;

	float Steps = 64.0;
	float StepLength = length( Ray ) / Steps;
	vec3 StepVec = Ray / Steps;
	vec4 InScattering = vec4( 0.0 );

	vec3  CurPoint = CameraPos;
	float CurDist  = 0.0;
	float Albedo = 2.0;
	float Tau = 0.02;
	float Phi = 32.0;
	mat4 Transform = in_ShadowMatrix[0];
	for ( int i = 0; i != int(Steps); i++ )
	{
		CurPoint += StepVec;
		CurDist  += StepLength;
		vec3 LightDirection = CurPoint - u_LightPosition.xyz;
#if defined( USE_VSDCT )
		vec2 IndirectTC = GetShadowTC( normalize( LightDirection ) );
		vec4 Light = texture( Texture7, IndirectTC );
#else
		vec4 Light = texture( Texture11, normalize( LightDirection ) );
#endif
		float LightD = Unpack( Light ) * 512.0;
		float D = length( LightDirection );
		float DepthBias = u_LightCastShadow.z;
		// point on the ray is not in shadow
		if ( LightD >= D + DepthBias && LightD > 0.0 )
		{
			float DistToLight = length( u_LightPosition.xyz - CurPoint );
			float Lin = exp( -DistToLight * Tau ) * Phi/4.0/3.14159/DistToLight/DistToLight;
			float Li  = Lin * Tau * Albedo;
			InScattering += Li * exp( -CurDist * Tau ) * CurDist;
		}
	}

	return InScattering;
}
