//

float ComputeDirectionalLightShadow()
{
	// shadow map
	vec4 ShadowCoords4 = v_ShadowMapCoord / v_ShadowMapCoord.w;

	if ( ShadowCoords4.w > 0.0 && u_ReceiveShadow.x > 0.5 && u_LightCastShadow.x > 0.5 )
	{
		vec2 ShadowCoord = vec2( ShadowCoords4 );

		float ShadowSample = texture( Texture7, ShadowCoord ).x;
		float DepthBias = u_LightCastShadow.z;

		// intensity of the shadow
		float ShadowCoef = u_LightCastShadow.y;

		if ( ShadowCoords4.z + DepthBias >= ShadowSample ) return ShadowCoef;
	}

	return 1.0;
}

#if defined( USE_VSDCT )
// Virtual Shadow Depth Cube Texture (VDSCT)
vec2 GetShadowTC( vec3 Dir )
{
	// OpenGL 4.4 Core Profile Specification
	// 	8.13 Cube Map Texture Selection
	// http://www.opengl.org/registry/doc/glspec44.core.pdf

	// Brute force dwells here, BEWARE!!!
	float Sc;
	float Tc;
	float Ma;
	float FaceIndex;

	float rx = Dir.x;
	float ry = Dir.y;
	float rz = Dir.z;

	vec3 adir = abs(Dir);
	Ma = max( max( adir.x, adir.y ), adir.z );

	// -X +X ok
	if ( adir.x > adir.y && adir.x > adir.z )
	{
		Sc = ( rx > 0.0 ) ? rz : -rz;
		Tc = ry;
		FaceIndex = ( rx > 0.0 ) ? 0.0 : 1.0;
	}
	// -Y +Y ok
	else if ( adir.y > adir.x && adir.y > adir.z )
	{
		Sc = rx;
		Tc = ( ry > 0.0 ) ? rz : -rz;
		FaceIndex = ( ry > 0.0 ) ? 2.0 : 3.0;
	}
	// -Z +Z ok
	else
	{
		Sc = ( rz > 0.0 ) ? -rx : rx;
		Tc = ry;
		FaceIndex = ( rz > 0.0 ) ? 4.0 : 5.0;
	}
	
	float s = 0.5 * ( Sc / Ma + 1.0 );
	float t = 0.5 * ( Tc / Ma + 1.0 );

/*
	Calculate texture coordinates for faces in our 3x2 layout:
		0   1   2
		3   4   5
*/
	s = s / 3.0;
	t = t / 2.0;

	float Flr = floor(FaceIndex / 3.0);
	// Knuth operator
	float Rmd = FaceIndex - (3.0 * Flr);

	s += Rmd / 3.0;
	t += Flr / 2.0;

	return vec2( s, t );
}
#endif

float ComputePointLightShadow()
{
	vec3 LightDirection = v_WorldPosition - u_LightPosition.xyz;

#if defined( USE_VSDCT )
	vec2 IndirectTC = GetShadowTC( normalize( LightDirection ) );
	vec4 Light = texture( Texture7, IndirectTC );
#else
	// fetch shadow cube map
	vec4 Light = texture( Texture11, normalize( LightDirection ) );
#endif

	float LightD = Unpack( Light ) * 512.0;
	float D = length( LightDirection );

	float DepthBias = u_LightCastShadow.z;

	// intensity of the shadow
	float ShadowCoef = u_LightCastShadow.y;

	return ( LightD < D + DepthBias ) ? ShadowCoef : 1.0;
}
