//

const float PI = 3.14159;

uniform float cvar_Engine_EngineTime;

uniform float u_NumWaves;

uniform float u_Amplitude[8];
uniform float u_Wavelength[8];
uniform float u_Speed[8];
uniform vec4  u_Direction[8];

float Wave( int i, float x, float y )
{
	float Frequency = 2 * PI / u_Wavelength[i];
	float Phase = u_Speed[i] * Frequency;
	float Theta = dot( u_Direction[i].xy, vec2( x, y ) );
	return u_Amplitude[i] * sin( Theta * Frequency + cvar_Engine_EngineTime * Phase );
}

float WaveHeight( float x, float y )
{
	float Height = 0.0;

	for ( int i = 0; i < int( u_NumWaves ); ++i )
	{
		Height += Wave( i, x, y );
	}

	return Height;
}

float dWavedx( int i, float x, float y )
{
	float Frequency = 2 * PI / u_Wavelength[i];
	float Phase = u_Speed[i] * Frequency;
	float Theta = dot( u_Direction[i].xy, vec2( x, y ) );
	float A = u_Amplitude[i] * u_Direction[i].x * Frequency;
	return A * cos( Theta * Frequency + cvar_Engine_EngineTime * Phase );
}

float dWavedy( int i, float x, float y )
{
	float Frequency = 2 * PI / u_Wavelength[i];
	float Phase = u_Speed[i] * Frequency;
	float Theta = dot( u_Direction[i].xy, vec2( x, y ) );
	float A = u_Amplitude[i] * u_Direction[i].y * Frequency;
	return A * cos( Theta * Frequency + cvar_Engine_EngineTime * Phase );
}

vec3 WaveNormal( float x, float y )
{
	float dx = 0.0;
	float dy = 0.0;

	for ( int i = 0; i < int( u_NumWaves ); ++i )
	{
		dx += dWavedx( i, x, y );
		dy += dWavedy( i, x, y );
	}

	vec3 n = vec3( -dx, -dy, 1.0 );
	return normalize( n );
}
