/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;

out vec4 Pos;
out vec3 EyeRay_Origin;
out vec3 EyeRay_Dir;

void main()
{
	const vec4 VOLUME_SIZE = 0.2 * vec4( 1.0, 1.0, 1.0, 1.0 );

   vec4 EyePos = in_Vertex * VOLUME_SIZE;

   Pos = in_ModelViewProjectionMatrix * EyePos;

   gl_Position = in_ModelViewProjectionMatrix * in_Vertex;

   EyeRay_Origin = ( in_ModelViewMatrixInverse * vec4(0.0, 0.0, 0.0, 1.0) ).xyz;
   // direction
   EyeRay_Dir = EyePos.xyz - EyeRay_Origin;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

#include "MaterialSystem/Noise/noise3D.glsl"

uniform sampler3D Texture1;

in vec4  Pos;
in vec3  EyeRay_Origin;
in vec3  EyeRay_Dir;

out vec4 out_FragColor;

#include "MaterialSystem/ShaderLib/VolumeRendering.sp"

const float CUT_OFF_LOW = 0.01;

float JitterPos( vec3 Vec )
{
#ifdef AMD
   return 0.0;
#else
   float Noise = ( snoise( 0.05*(256.0 * Vec) ) - 0.5 ) / 256.0;

   return Noise;
#endif
}

vec4 TraceMIP( vec3 FrontPos, vec3 DeltaDir, int Steps, float MaxLength )
{
   vec3 Vec       = FrontPos + JitterPos( FrontPos ) * DeltaDir;

   float LengthAcc = 0.0;
   float StepSize  = length( DeltaDir );

   float MaxScalar = 0.0;
   vec3 MaxVec    = Vec;

   for(int i = 0; i < Steps; i++)
   {
      float Scalar = textureLod( Texture1, Vec, 0.0 ).r;
                                                       
#ifdef ENABLE_CLIPPLANE
      if ( !IsClippedPos(Vec) )
#endif
      if ( Scalar > MaxScalar && Scalar > CUT_OFF_LOW )
      {
         MaxScalar = Scalar;
         MaxVec = Vec;
      }

      Vec += DeltaDir;
      LengthAcc += StepSize;

      if (LengthAcc >= MaxLength ) break;
   }

   vec4 Color = vec4( MaxScalar * ( MaxLength / LengthAcc ) );

   Color.a = MaxScalar;

   return Color;
}

void main()
{
	const float TRACING_STEPS = 256.0;

   // calculate ray intersection with bounding box
   float tnear, tfar;
   Ray EyeRay;
   EyeRay.o = EyeRay_Origin;
   EyeRay.d = normalize( EyeRay_Dir );
   bool hit = IntersectBox( EyeRay, BBoxMin, BBoxMax, tnear, tfar);
   if (!hit) discard;
   if (tnear < 0.0) tnear = 0.0;

   int   Steps = int(TRACING_STEPS);

   vec2 Coord = ( ( Pos.xy / Pos.w ) + 1.0 ) / 2.0;

   vec3 BackPosition  = EyeRay.o + EyeRay.d * tfar;
   vec3 FrontPosition = EyeRay.o + EyeRay.d * tnear;

   // convert to texture space
   BackPosition  = BackPosition  * 0.5 + vec3( 0.5 );
   FrontPosition = FrontPosition * 0.5 + vec3( 0.5 );

   vec3 Dir = BackPosition - FrontPosition;
   float Length = length( Dir );

   vec3 DeltaDir = Dir / Steps;

	//

   vec4 Color = TraceMIP( FrontPosition, DeltaDir, Steps, Length );

   out_FragColor = Color;
}
