/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;
in vec3 in_Normal;
in vec2 in_Bones;

out vec2 v_TexCoord;
out vec3 v_WorldNormal;
out vec3 v_WorldPosition;
out vec4 v_ProjectedVertex;

#include MaterialSystem/ShaderLib/Uniforms_Vertex.sp

#include MaterialSystem/ShaderLib/Projections.sp
#include MaterialSystem/ShaderLib/Waves.sp

void main()
{
   vec4 Position = in_Vertex;

	Position += vec4( 0.0, 0.0, WaveHeight( in_Vertex.x, in_Vertex.y), 0.0 );

   gl_Position = in_ModelViewProjectionMatrix * Position;

   v_TexCoord = in_TexCoord.xy;
	v_WorldNormal = WaveNormal( in_Vertex.x, in_Vertex.y);
	v_WorldPosition = ( in_ModelMatrix * Position ).xyz;
	v_ProjectedVertex = GetProjScaleBiasMat() * in_ModelViewProjectionMatrix * Position;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 v_TexCoord;
in vec3 v_WorldNormal;
in vec3 v_WorldPosition;
in vec4 v_ProjectedVertex;

out vec4 out_FragColor;

#include MaterialSystem/ShaderLib/Uniforms_Fragment.sp

void main()
{
	vec4 FinalColor = texture( Texture1, v_TexCoord ) + u_AmbientColor;

	float Reflectivity = u_MaterialParams.y;

	if ( Reflectivity > 0.0 )
	{
		float Eta = u_MaterialShading.y;

		vec3 WorldNormal = normalize(v_WorldNormal);

		vec3 CameraPos = in_ViewMatrixInverse[3].xyz;
		vec3 Eye       = normalize( CameraPos - v_WorldPosition );

		float E_dot_N = dot( Eye, WorldNormal );

		float FresnelTerm = clamp( Eta + (1.0 - Eta) * pow( max(0.0, 1.0 - E_dot_N), 5.0 ), 0.0, 1.0 );

		float Strength = 0.75;

		vec4 Ofs = vec4( Strength * WorldNormal.xy / v_ProjectedVertex.z, 0.0, 0.0 );

		vec4 ReflectionColor = textureProj( Texture5, v_ProjectedVertex + Ofs );

		FinalColor = mix( FinalColor, ReflectionColor * Reflectivity, FresnelTerm );
	}

	out_FragColor = FinalColor;
}
