/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;
in vec3 in_Normal;
in vec2 in_Bones;

out vec2 v_TexCoord;
out vec4 v_ShadowMapCoord;
out vec3 v_WorldNormal;
out vec3 v_WorldPosition;
out vec4 v_ProjectedVertex;

#include MaterialSystem/ShaderLib/Uniforms_Vertex.sp

#include MaterialSystem/ShaderLib/Projections.sp
#include MaterialSystem/ShaderLib/Waves.sp

void main()
{
   vec4 Position = in_Vertex;

	Position += vec4( 0.0, 0.0, WaveHeight( in_Vertex.x, in_Vertex.y), 0.0 );

   gl_Position = in_ModelViewProjectionMatrix * Position;

   v_TexCoord = in_TexCoord.xy;
	v_ShadowMapCoord = in_ShadowMatrix[0] * in_ModelMatrix * Position;
	v_WorldNormal = WaveNormal( in_Vertex.x, in_Vertex.y);
	v_WorldPosition = ( in_ModelMatrix * Position ).xyz;
	v_ProjectedVertex = GetProjScaleBiasMat() * in_ModelViewProjectionMatrix * Position;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 v_TexCoord;
in vec4 v_ShadowMapCoord;
in vec3 v_WorldNormal;
in vec3 v_WorldPosition;
in vec4 v_ProjectedVertex;

out vec4 out_FragColor;

#include MaterialSystem/ShaderLib/Uniforms_Fragment.sp

#include MaterialSystem/ShaderLib/Pack.sp
#include MaterialSystem/ShaderLib/ShadowUtils.sp
#include MaterialSystem/ShaderLib/Projections.sp
#include MaterialSystem/ShaderLib/LightingUtils.sp
#include MaterialSystem/ShaderLib/Lighting_Lambert.sp
#include MaterialSystem/ShaderLib/Lighting_OrenNayar.sp
#include MaterialSystem/ShaderLib/Lighting_CookTorrance.sp
#include MaterialSystem/ShaderLib/Shadows.sp

void main()
{
	vec4 Kd = texture( Texture2, v_TexCoord ) + u_DiffuseColor;
	vec4 Ks = texture( Texture3, v_TexCoord ) + u_SpecularColor;

	// compute some fake lighting if there are no lights in the scene
 	vec4 FinalColor = Kd;

	float LightType   = u_LightAttenuation.w;

	const float LIGHT_TYPE_NONE        = 0.0;
	const float LIGHT_TYPE_DIRECTIONAL = 1.0;
	const float LIGHT_TYPE_SPOT        = 2.0;
	const float LIGHT_TYPE_POINT       = 3.0;

	vec3 WorldNormal = normalize(v_WorldNormal);

	if ( LightType == LIGHT_TYPE_DIRECTIONAL )
	{
		FinalColor = ComputeDirectionalLight_CookTorrance( WorldNormal, Kd, Ks );
		FinalColor *= ComputeDirectionalLightShadow();
	}
	else if ( LightType == LIGHT_TYPE_SPOT )
	{
		FinalColor = ComputeSpotLight_CookTorrance( WorldNormal, Kd, Ks );
		FinalColor *= ComputeDirectionalLightShadow();
	}
	else if ( LightType == LIGHT_TYPE_POINT )
	{
		FinalColor = ComputePointLight_CookTorrance( WorldNormal, Kd, Ks );
		FinalColor *= ComputePointLightShadow();
	}

	float Transparency = u_MaterialParams.x;

	FinalColor.a = 1.0 - Transparency;

	out_FragColor = FinalColor;
}
