/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;

// start, end, color, thickness
uniform mat4 LineParams;

uniform mat4 ModelViewMatrix;
uniform mat4 ProjectionMatrix;

out vec2 TexCoord;

void main()
{
	vec4 LineStart  = LineParams[0];
	vec4 LineEnd    = LineParams[1];
	float Thickness = LineParams[3].x;

	mat4 MVP = ProjectionMatrix * ModelViewMatrix;

	vec4 Start = MVP * LineStart;
	vec4 End   = MVP * LineEnd;

//	vec2 Start2D = Start.xy / Start.w;
//	vec2 End2D   = End.xy / End.w;

	vec4 D = Thickness * normalize(End - Start);
	vec4 A = vec4( -D.y, D.x, 0.0, 0.0 );

	int Index = int( in_Vertex.x );

/**
	/// This code does not work on Tegra, constant offsets are required

	vec4 Offsets[8];

	// x * Pt1 + y * Pt2 + z * D + w * A       // vec2( z, w ) * Thickness
	Offsets[0] = vec4( 1.0, 0.0, -1.0,  1.0 );
	Offsets[1] = vec4( 1.0, 0.0,  0.0,  1.0 );
	Offsets[2] = vec4( 0.0, 1.0,  0.0,  1.0 );
	Offsets[3] = vec4( 0.0, 1.0,  1.0,  1.0 );
	Offsets[4] = vec4( 0.0, 1.0,  1.0, -1.0 );
	Offsets[5] = vec4( 0.0, 1.0,  0.0, -1.0 );
	Offsets[6] = vec4( 1.0, 0.0,  0.0, -1.0 );
	Offsets[7] = vec4( 1.0, 0.0, -1.0, -1.0 );

	vec4 Off = Offsets[ Index ];

	vec4 Position = Off.x * Start + Off.y * End + Off.z * D + Off.w * A;
*/

/** Workaround for Tegra */

	vec4 Position = vec4(0);

	Position = ( (Index < 2) || ( Index > 5) ) ? Start : End;

	Position += (Index == 0 || Index == 7) ? -D : ( (Index == 3 || Index == 4) ? D : vec4(0) );

	Position += (Index < 4) ? A : -A;

/** End Of Workaround */

	TexCoord = in_TexCoord.xy;

	gl_Position = Position;
}

/*FRAGMENT_PROGRAM*/

in vec2 TexCoord;

uniform mat4 LineParams;

out vec4 out_FragColor;

void main()
{
	float Thickness = LineParams[3].x;

	float Coef = Thickness / 3.0;

	vec2 Dist  = ( TexCoord - vec2( 0.5 ) ) / Coef;
	vec2 Dist2 = -Dist*Dist;

	float Alpha = exp( Dist2.x + Dist2.y );

	out_FragColor = LineParams[2] * vec4( 1.0, 1.0, 1.0, Alpha );
}
