/*VERTEX_PROGRAM*/

#include Layout.sp

in vec4 in_Vertex;
in vec4 in_TexCoord;
in vec3 in_Normal; // TTL, lifetime, size
in vec4 in_Color;

out vec2 TexCoord0;
out vec3 TexCoord1;
out vec4 TexCoord2;

void main()
{
   vec3 Position = in_Vertex.xyz;

   vec3 X = vec3(in_ModelViewMatrix[0][0], in_ModelViewMatrix[1][0], in_ModelViewMatrix[2][0]);
   vec3 Y = vec3(in_ModelViewMatrix[0][1], in_ModelViewMatrix[1][1], in_ModelViewMatrix[2][1]);
  
   float SizeX = in_Normal.z;
   float SizeY = in_Normal.z;

   vec3 A1 = -X*SizeX - Y*SizeY;
   vec3 B1 =  X*SizeX - Y*SizeY;
   vec3 C1 =  X*SizeX + Y*SizeY;
   vec3 D1 = -X*SizeX + Y*SizeY;

   if ( in_TexCoord.xy == vec2(0.0, 0.0) ) Position += A1;
   if ( in_TexCoord.xy == vec2(1.0, 0.0) ) Position += B1;
   if ( in_TexCoord.xy == vec2(1.0, 1.0) ) Position += C1;
   if ( in_TexCoord.xy == vec2(0.0, 1.0) ) Position += D1;

   vec4 TransformedPos = in_ModelViewProjectionMatrix * vec4(Position, 1.0);

   gl_Position = TransformedPos;

   TexCoord0 = in_TexCoord.xy;
   TexCoord1 = in_Normal;
   TexCoord2 = in_Color;
}

/*FRAGMENT_PROGRAM*/

#include Layout.sp

in vec2 TexCoord0;
in vec3 TexCoord1; /* TTL, LifeTime, Size */
in vec4 TexCoord2; /* RGBA overlay */

uniform sampler2D Texture0; /* Color texture */

out vec4 out_FragColor;
                                              
void main()
{
   vec4 ParticleColor = vec4( texture(Texture0, TexCoord0) );

   float TTL      = TexCoord1.x;
   float LifeTime = TexCoord1.y;

	out_FragColor = TexCoord2 * ParticleColor; //* TTL / LifeTime;
}
